@@ -197,7 +197,7 @@ typedef struct SshMappingVLRec
 /* Calculates a hash value from a buffer. */
 
 unsigned long ssh_default_hash_function(const void *key,
-					size_t key_length)
+                                        size_t key_length)
 {
   size_t i;
   unsigned long hash;
@@ -212,21 +212,32 @@ unsigned long ssh_default_hash_function(const void *key,
 }
 
 unsigned long ssh_default_hash_crc_function(const void *key,
-					    size_t key_length)
+                                            size_t key_length)
 {
   return crc32_buffer((const unsigned char *)key,
-		      (unsigned int)(key_length & 0xffffffff));
+                      (unsigned int)(key_length & 0xffffffff));
 }
 
 /* Default compare function, uses memcmp directly. */
 
 int ssh_default_compare_function(const void *key1,
-				 const void *key2,
-				 size_t key_length)
+                                 const void *key2,
+                                 size_t key_length)
 {
   return memcmp(key1, key2, key_length);
 }
 
+/* Default destructor function, uses ssh_xfree */
+void ssh_default_destructor_function(void *key,
+                                     size_t key_length,
+                                     void *value,
+                                     size_t value_length)
+{
+  ssh_xfree(key);
+  ssh_xfree(value);
+}
+
+
 /* Array of predefined mapping types. This array is searched
    in ssh_mapping_allocated with the given SshMappingType. */
 
@@ -307,12 +318,12 @@ void ssh_mapping_fatal(void)
 #define MODULO_ADD(size, h1, h2)             \
   (h1) = ((h1) + (h2)) & ((size) - 1);       
 #endif /* PRIME_TABLE */
-	
+        
 
 /* Allocate space for the hash table. The size is given as argument. */
 
 SshMappingEntry *ssh_mapping_allocate_hashtable(SshMapping set,
-						unsigned int size)
+                                                unsigned int size)
 {
   SshMappingEntry *t = NULL;
   
@@ -330,7 +341,7 @@ SshMappingEntry *ssh_mapping_allocate_hashtable(SshMapping set,
 /* Allocate space for hash table with variable length keys. */
 
 SshMappingEntryVL *ssh_mapping_allocate_hashtable_vl(SshMappingVL set,
-						     unsigned int size)
+                                                     unsigned int size)
 {
   SshMappingEntryVL *t = NULL;
   
@@ -355,20 +366,20 @@ void ssh_mapping_free_hashtable(SshMapping set)
   for (i = 0; i < set->size; i++)
     {
       if (set->t[i].h > 1)
-	{
-	  /* If the cell is not empty or semi empty, call the
-	     destructor for the key and the value. If no destructor is
-	     defined, just free the value. This also frees the memory
-	     possibly allocated for key. */
-	  if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	    {
-	      if (set->destructor_function)
-		set->destructor_function(set->t[i].key.p, set->key_length,
-					 set->t[i].value, set->value_length);
-	    }
-	  else
-	    ssh_xfree(set->t[i].value);
-	}
+        {
+          /* If the cell is not empty or semi empty, call the
+             destructor for the key and the value. If no destructor is
+             defined, just free the value. This also frees the memory
+             possibly allocated for key. */
+          if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
+            {
+              if (set->destructor_function)
+                set->destructor_function(set->t[i].key.p, set->key_length,
+                                         set->t[i].value, set->value_length);
+            }
+          else
+            ssh_xfree(set->t[i].value);
+        }
     }
   ssh_xfree(set->t);
 }
@@ -384,21 +395,21 @@ void ssh_mapping_free_hashtable_vl(SshMappingVL set)
   for (i = 0; i < set->size; i++)
     {
       if (set->t[i].h > 1)
-	{
-	  if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	    {
-	      if (set->destructor_function)
-		set->destructor_function(set->t[i].key.p, set->key_length,
-					 set->t[i].value, set->value_length);
-	    }
-	  else
-	    {
-	      /* Possible key and value must be deallocated separately. */
-	      if (!(set->flags & SSH_MAPPING_FL_INTEGER_KEY))
-		ssh_xfree(set->t[i].key.p);
-	      ssh_xfree(set->t[i].value);
-	    }
-	}
+        {
+          if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
+            {
+              if (set->destructor_function)
+                set->destructor_function(set->t[i].key.p, set->key_length,
+                                         set->t[i].value, set->value_length);
+            }
+          else
+            {
+              /* Possible key and value must be deallocated separately. */
+              if (!(set->flags & SSH_MAPPING_FL_INTEGER_KEY))
+                ssh_xfree(set->t[i].key.p);
+              ssh_xfree(set->t[i].value);
+            }
+        }
     }
   ssh_xfree(set->t);
 }
@@ -408,10 +419,10 @@ void ssh_mapping_free_hashtable_vl(SshMappingVL set)
    values. */
 
 SshMapping ssh_mapping_allocate_vl(SshMappingFlags flags,
-				   SshMapHashFunctionProc hash_func,
-				   SshMapHashCompareProc compare_func,
-				   size_t key_length,
-				   size_t value_length)
+                                   SshMapHashFunctionProc hash_func,
+                                   SshMapHashCompareProc compare_func,
+                                   size_t key_length,
+                                   size_t value_length)
 {
   SshMappingVL set;
 
@@ -464,11 +475,11 @@ SshMapping ssh_mapping_allocate_vl(SshMappingFlags flags,
    one for hash function and other for compare. Returns pointer to
    allocated SshMapping set or NULL if an error occured. */
 SshMapping ssh_mapping_allocate_with_func(SshMappingFlags flags,
-					  SshMapHashFunctionProc hash_func,
-					  SshMapHashCompareProc compare_func,
-					  SshMapHashDestructorProc destr_func,
-					  size_t key_length,
-					  size_t value_length)
+                                          SshMapHashFunctionProc hash_func,
+                                          SshMapHashCompareProc compare_func,
+                                          SshMapHashDestructorProc destr_func,
+                                          size_t key_length,
+                                          size_t value_length)
 {
   SshMapping set;
 
@@ -477,7 +488,7 @@ SshMapping ssh_mapping_allocate_with_func(SshMappingFlags flags,
   if (flags & SSH_MAPPING_FL_VARIABLE_LENGTH)
     {
       set = ssh_mapping_allocate_vl(flags, hash_func, compare_func,
-				    key_length, value_length);
+                                    key_length, value_length);
       return set;
     }
 
@@ -528,7 +539,7 @@ SshMapping ssh_mapping_allocate_with_func(SshMappingFlags flags,
    of both the key and value associated with it. This function returns
    pointer to allocated SshMapping set or NULL in case of error. */
 SshMapping ssh_mapping_allocate(SshMappingType mapping_type,
-				size_t key_length, size_t value_length)
+                                size_t key_length, size_t value_length)
 {
   SshMapping set;
   int i;
@@ -539,7 +550,7 @@ SshMapping ssh_mapping_allocate(SshMappingType mapping_type,
        i++)
     {
       if (ssh_mapping_type_array[i].type == mapping_type)
-	break;
+        break;
     }
   /* If the requested type was not in array, fail by returing NULL. */
   if (ssh_mapping_type_array[i].type == SSH_MAPPING_TYPE_END_OF_ARRAY)
@@ -547,10 +558,10 @@ SshMapping ssh_mapping_allocate(SshMappingType mapping_type,
 
   /* Allocate actual set with ssh_mapping_with_func(). */
   set = ssh_mapping_allocate_with_func(ssh_mapping_type_array[i].flags,
-				       ssh_mapping_type_array[i].hash_function,
-				       ssh_mapping_type_array[i].compare,
-				       ssh_mapping_type_array[i].destructor,
-				       key_length, value_length);
+                                       ssh_mapping_type_array[i].hash_function,
+                                       ssh_mapping_type_array[i].compare,
+                                       ssh_mapping_type_array[i].destructor,
+                                       key_length, value_length);
 
   /* Return the allocated set. */
   return set;
@@ -571,8 +582,8 @@ void ssh_mapping_free(SshMapping set)
 
       /* If there is a remove value stores in context, free it here. */
       if (vlset->remove_value &&
-	  !(vlset->flags & SSH_MAPPING_FL_STORE_POINTERS))
-	ssh_xfree(vlset->cached_value);
+          !(vlset->flags & SSH_MAPPING_FL_STORE_POINTERS))
+        ssh_xfree(vlset->cached_value);
       
       ssh_xfree(vlset);
       return;
@@ -609,8 +620,8 @@ void ssh_mapping_clear(SshMapping set)
 
       /* If there is a remove value stores in context, free it here. */
       if (vlset->remove_value &&
-	  !(vlset->flags & SSH_MAPPING_FL_STORE_POINTERS))
-	ssh_xfree(vlset->cached_value);
+          !(vlset->flags & SSH_MAPPING_FL_STORE_POINTERS))
+        ssh_xfree(vlset->cached_value);
       vlset->cached_value = NULL;
       
       /* Allocate space for a new mapping. */
@@ -649,13 +660,13 @@ void ssh_mapping_refresh(SshMapping set, int new_size)
   for (i = 0; i < set->size; i++)
     if ((unsigned long)set->t[i].h > 1)
       {
-	h = set->t[i].h;
-	REDUCE_HASH(new_size, h, h1, h2);
-	while (nt[h1].h != 0)
-	  {
-	    MODULO_ADD(new_size, h1, h2);
-	  }
-	memcpy(&nt[h1], &set->t[i], sizeof(SshMappingEntry));
+        h = set->t[i].h;
+        REDUCE_HASH(new_size, h, h1, h2);
+        while (nt[h1].h != 0)
+          {
+            MODULO_ADD(new_size, h1, h2);
+          }
+        memcpy(&nt[h1], &set->t[i], sizeof(SshMappingEntry));
       }
 
   /* Free the old array and start using the new one. */
@@ -670,7 +681,7 @@ void ssh_mapping_refresh(SshMapping set, int new_size)
 /* Copy only pointers to mapping entry. */
 
 static void ssh_mapping_set_entry(SshMapping set, unsigned long i,
-				  void *key, void *value)
+                                  void *key, void *value)
 {
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     set->t[i].key.i = *((unsigned long *)key);
@@ -687,7 +698,7 @@ static void ssh_mapping_set_entry(SshMapping set, unsigned long i,
 (((set)->flags & SSH_MAPPING_FL_INTEGER_KEY) ?              \
  ((set)->t[idx].key.i == *((unsigned long *)keyp)) :      \
  ((set)->compare_function((keyp), (set)->t[(idx)].key.p,  \
-			  (set)->key_length) == 0))
+                          (set)->key_length) == 0))
 
 /* Adds a mapping for a key. Duplicate pages are stored only once.  If
    mapping for this key already exists, the old value is replaced with
@@ -717,10 +728,10 @@ Boolean ssh_mapping_put(SshMapping set, void *key, void *value)
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     {
       if (set->hash_function)
-	h = set->hash_function(key, sizeof(unsigned long));
+        h = set->hash_function(key, sizeof(unsigned long));
       else
-	/* In case of integer keys, the key can be used directly. */
-	h = *((unsigned long *)key);
+        /* In case of integer keys, the key can be used directly. */
+        h = *((unsigned long *)key);
     }
   else
     h = set->hash_function(key, set->key_length);
@@ -730,7 +741,7 @@ Boolean ssh_mapping_put(SshMapping set, void *key, void *value)
     h += 2;
 
   REDUCE_HASH(set->size, h, h1, h2);
-	  
+          
   start = h1;
   
   /* i is the index of the first entry where the mapping can be put if 
@@ -739,29 +750,29 @@ Boolean ssh_mapping_put(SshMapping set, void *key, void *value)
     {
       flag = set->t[h1].h;
       if (flag < 2)
-	{
-	  if (i < 0)
-	    i = h1;
-	  if (flag == 0)
-	    break;
-	  semi_empty_count++;
-	}
+        {
+          if (i < 0)
+            i = h1;
+          if (flag == 0)
+            break;
+          semi_empty_count++;
+        }
       else
-	{
-	  if (flag == h &&
-	      SSH_MAPPING_COMPARE_KEY(set, h1, key))
-	    {
-	      i = h1;
-	      break;
-	    }
-	}
+        {
+          if (flag == h &&
+              SSH_MAPPING_COMPARE_KEY(set, h1, key))
+            {
+              i = h1;
+              break;
+            }
+        }
       MODULO_ADD(set->size, h1, h2);
       if (h1 == start)
-	{
-	  if (i != -1)
-	    break;
-	  ssh_mapping_fatal();
-	}
+        {
+          if (i != -1)
+            break;
+          ssh_mapping_fatal();
+        }
     }
 
   set->t[i].h = h;
@@ -769,32 +780,32 @@ Boolean ssh_mapping_put(SshMapping set, void *key, void *value)
   if (flag != h)
     {      
       if (i == (int) h1)
-	set->empties--;
+        set->empties--;
       else
-	set->semi_empties--;
+        set->semi_empties--;
 
       /* Allocate a new buffer for key and it's value and copy
-	 the data. */
+         the data. */
       if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	ssh_mapping_set_entry(set, i, key, value);
+        ssh_mapping_set_entry(set, i, key, value);
       else
-	{
-	  /* Allocate space for value and possibly for the key also. */
-	  set->t[i].value = ssh_xmalloc(set->key_length + set->value_length);
+        {
+          /* Allocate space for value and possibly for the key also. */
+          set->t[i].value = ssh_xmalloc(set->key_length + set->value_length);
 
-	  /* Copy the value to the buffer. */
-	  memcpy(set->t[i].value, value, set->value_length);
+          /* Copy the value to the buffer. */
+          memcpy(set->t[i].value, value, set->value_length);
 
-	  /* Copy the key. */
-	  if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
-	    set->t[i].key.i = *((unsigned long *)key);
-	  else
-	    {
-	      set->t[i].key.p = (unsigned char *) set->t[i].value +
-		set->value_length;
-	      memcpy(set->t[i].key.p, key, set->key_length);
-	    }
-	}
+          /* Copy the key. */
+          if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
+            set->t[i].key.i = *((unsigned long *)key);
+          else
+            {
+              set->t[i].key.p = (unsigned char *) set->t[i].value +
+                set->value_length;
+              memcpy(set->t[i].key.p, key, set->key_length);
+            }
+        }
       return FALSE;
     }
 
@@ -812,7 +823,7 @@ Boolean ssh_mapping_put(SshMapping set, void *key, void *value)
    FALSE otherwise.  If value_return is not NULL, copies also the mapped
    value to buffer pointer in it. */
 Boolean ssh_mapping_get(const SshMapping set, const void *key,
-			void *value_return)
+                        void *value_return)
 {
   unsigned long h, h1, flag;
   unsigned long h2, start;
@@ -828,10 +839,10 @@ Boolean ssh_mapping_get(const SshMapping set, const void *key,
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     {
       if (set->hash_function)
-	h = set->hash_function(key, sizeof(unsigned long));
+        h = set->hash_function(key, sizeof(unsigned long));
       else
-	/* In case of integer keys, the key can be used directly. */
-	h = *((unsigned long *)key);
+        /* In case of integer keys, the key can be used directly. */
+        h = *((unsigned long *)key);
     }
   else
     h = set->hash_function(key, set->key_length);
@@ -848,18 +859,18 @@ Boolean ssh_mapping_get(const SshMapping set, const void *key,
       SSH_MAPPING_COMPARE_KEY(set, h1, key))
     {
       if (value_return)
-	{
-	  if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	    *((void **)value_return) = set->t[h1].value;
-	  else
-	    memcpy(value_return, set->t[h1].value, set->value_length);
-	}
+        {
+          if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
+            *((void **)value_return) = set->t[h1].value;
+          else
+            memcpy(value_return, set->t[h1].value, set->value_length);
+        }
       return TRUE;
     }
   else
     {
       if (flag == 0)
-	return FALSE;
+        return FALSE;
     }
 
   start = h1;
@@ -869,21 +880,21 @@ Boolean ssh_mapping_get(const SshMapping set, const void *key,
       MODULO_ADD(set->size, h1, h2);
       flag = set->t[h1].h;
       if (flag == h &&
-	  SSH_MAPPING_COMPARE_KEY(set, h1, key))
-	{
-	  if (value_return)
-	    {
-	      if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-		*((void **)value_return) = set->t[h1].value;
-	      else
-		memcpy(value_return, set->t[h1].value, set->value_length);
-	    }
-	  return TRUE;
-	}
+          SSH_MAPPING_COMPARE_KEY(set, h1, key))
+        {
+          if (value_return)
+            {
+              if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
+                *((void **)value_return) = set->t[h1].value;
+              else
+                memcpy(value_return, set->t[h1].value, set->value_length);
+            }
+          return TRUE;
+        }
       else if (flag == 0)
-	return FALSE;
+        return FALSE;
       if (h1 == start)
-	return FALSE;     
+        return FALSE;     
     }  
 }
 
@@ -893,10 +904,23 @@ Boolean ssh_mapping_get(const SshMapping set, const void *key,
    if value_return is not NULL, also copies the old mapped value to
    it. */
 Boolean ssh_mapping_remove(SshMapping set, const void *key,
-			   void *value_return)
+                           void *value_return)
+{
+  return ssh_mapping_remove_key(set, key, value_return, NULL);
+}
+
+
+/* Same as ssh_mapping_remove, but if the mapping is STORE_POINTERS
+   type, this function also returns the original key pointer in
+   'key_return'.  Otherwise it is set to NULL. */
+Boolean ssh_mapping_remove_key(SshMapping set, const void *key,
+                               void *value_return, void *key_return)
 {
   unsigned long h, h1, h2;
   unsigned long flag, start;
+  
+  if (key_return != NULL)
+    *((void **)key_return) = NULL;
 
   /* This should not be used with variable length mappings. */
   if (set->flags & SSH_MAPPING_FL_VARIABLE_LENGTH)
@@ -904,15 +928,15 @@ Boolean ssh_mapping_remove(SshMapping set, const void *key,
   
   if (set->size == set->empties + set->semi_empties)
     return FALSE;  /* Empty set, not a member. */
-
-    /* Calculate a hash value for the key. */
+  
+  /* Calculate a hash value for the key. */
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     {
       if (set->hash_function)
-	h = set->hash_function(key, sizeof(unsigned long));
+        h = set->hash_function(key, sizeof(unsigned long));
       else
-	/* In case of integer keys, the key can be used directly. */
-	h = *((unsigned long *)key);
+        /* In case of integer keys, the key can be used directly. */
+        h = *((unsigned long *)key);
     }
   else
     h = set->hash_function(key, set->key_length);
@@ -929,13 +953,13 @@ Boolean ssh_mapping_remove(SshMapping set, const void *key,
     {
       flag = set->t[h1].h;
       if (flag == h &&
-	  SSH_MAPPING_COMPARE_KEY(set, h1, key))
-	break;
+          SSH_MAPPING_COMPARE_KEY(set, h1, key))
+        break;
       else if (flag == 0)
-	return FALSE;
+        return FALSE;
       MODULO_ADD(set->size, h1, h2);
       if (start == h1)
-	return FALSE;
+        return FALSE;
     }
 
   set->semi_empties++;
@@ -943,12 +967,16 @@ Boolean ssh_mapping_remove(SshMapping set, const void *key,
   if (value_return)
     {
       if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	*((void **)value_return) = set->t[h1].value;
+        {
+          *((void **)value_return) = set->t[h1].value;
+          if (key_return != NULL)
+            *((void **)key_return) = set->t[h1].key.p;
+        }
       else
-	{	  
-	  memcpy(value_return, set->t[h1].value, set->value_length);
-	  ssh_xfree(set->t[h1].value);
-	}
+        {
+          memcpy(value_return, set->t[h1].value, set->value_length);
+          ssh_xfree(set->t[h1].value);
+        }
     }
   
   /* Decrease the size of the hash table if it appears 
@@ -959,17 +987,17 @@ Boolean ssh_mapping_remove(SshMapping set, const void *key,
       set->semi_empties > (set->size >> 3))
     {
       if (set->size > SSH_MAPPING_MINIMUM_SIZE &&
-	  set->size - set->empties - set->semi_empties < (set->size >> 4))
-	{
+          set->size - set->empties - set->semi_empties < (set->size >> 4))
+        {
 #ifdef PRIME_TABLE
-	  set->size_shift--;
-	  ssh_mapping_refresh(set, prime_table[set->size_shift]);
+          set->size_shift--;
+          ssh_mapping_refresh(set, prime_table[set->size_shift]);
 #else
-	  ssh_mapping_refresh(set, set->size >> 1);
+          ssh_mapping_refresh(set, set->size >> 1);
 #endif /* PRIME_TABLE */
-	}
+        }
       else  
-	ssh_mapping_refresh(set, set->size);
+        ssh_mapping_refresh(set, set->size);
     }
   
   return TRUE;
@@ -1008,7 +1036,7 @@ void ssh_mapping_reset_index(SshMapping set)
 }
 
 Boolean ssh_mapping_get_next(SshMapping set, void *key_return,
-			     void *value_return)
+                             void *value_return)
 {
   unsigned long i;
 
@@ -1022,31 +1050,31 @@ Boolean ssh_mapping_get_next(SshMapping set, void *key_return,
   for (i = set->idx; i < set->size; i++)
     if (set->t[i].h > 1)
       {
-	set->idx = i + 1;
-	if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	  {
-	    if (key_return)
-	      {
-		if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
-		  *((unsigned long *)key_return) = set->t[i].key.i;
-		else
-		  *((void **)key_return) = set->t[i].key.p;
-	      }
-	    if (value_return)
-	      *((void **)value_return) = set->t[i].value;	    
-	    return TRUE;
-	  }
-	
-	if (key_return)
-	  {
-	    if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
-	      *((unsigned long *)key_return) = set->t[i].key.i;
-	    else
-	      memcpy(key_return, set->t[i].key.p, set->key_length);
-	  }
-	if (value_return)
-	  memcpy(value_return, set->t[i].value, set->value_length);
-	return TRUE;
+        set->idx = i + 1;
+        if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
+          {
+            if (key_return)
+              {
+                if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
+                  *((unsigned long *)key_return) = set->t[i].key.i;
+                else
+                  *((void **)key_return) = set->t[i].key.p;
+              }
+            if (value_return)
+              *((void **)value_return) = set->t[i].value;           
+            return TRUE;
+          }
+        
+        if (key_return)
+          {
+            if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
+              *((unsigned long *)key_return) = set->t[i].key.i;
+            else
+              memcpy(key_return, set->t[i].key.p, set->key_length);
+          }
+        if (value_return)
+          memcpy(value_return, set->t[i].value, set->value_length);
+        return TRUE;
       }
 
   return FALSE;
@@ -1071,13 +1099,13 @@ void ssh_mapping_refresh_vl(SshMappingVL set, int new_size)
   for (i = 0; i < set->size; i++)
     if ((unsigned long)set->t[i].h > 1)
       {
-	h = set->t[i].h;
-	REDUCE_HASH(new_size, h, h1, h2); 
-	while (nt[h1].h != 0)
-	  {
-	    MODULO_ADD(new_size, h1, h2);
-	  }
-	memcpy(&nt[h1], &set->t[i], sizeof(SshMappingEntryVL));
+        h = set->t[i].h;
+        REDUCE_HASH(new_size, h, h1, h2); 
+        while (nt[h1].h != 0)
+          {
+            MODULO_ADD(new_size, h1, h2);
+          }
+        memcpy(&nt[h1], &set->t[i], sizeof(SshMappingEntryVL));
       }
 
   /* Free the old array and start using the new one. */
@@ -1092,7 +1120,7 @@ void ssh_mapping_refresh_vl(SshMappingVL set, int new_size)
 /* Copy only pointers to mapping entry. */
 
 static void ssh_mapping_set_entry_vl(SshMappingVL set, unsigned long i,
-				     void *key, void *value)
+                                     void *key, void *value)
 {
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     set->t[i].key.i = *((unsigned long *)key);
@@ -1113,8 +1141,8 @@ static void ssh_mapping_set_entry_vl(SshMappingVL set, unsigned long i,
 /* Variable length version of ssh_mapping_put. In addition to
    key and value pointers this also needs their sizes. */
 Boolean ssh_mapping_put_vl(SshMapping normal_set,
-			   void *key, size_t key_length,
-			   void *value, size_t value_length)
+                           void *key, size_t key_length,
+                           void *value, size_t value_length)
 {
   SshMappingVL set = (SshMappingVL) normal_set;
   unsigned long h, h1, h2, start;
@@ -1138,9 +1166,9 @@ Boolean ssh_mapping_put_vl(SshMapping normal_set,
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     {
       if (set->hash_function)
-	h = set->hash_function(key, sizeof(unsigned long));
+        h = set->hash_function(key, sizeof(unsigned long));
       else
-	h = *((unsigned long *)key);
+        h = *((unsigned long *)key);
     }
   else
     h = set->hash_function(key, key_length);
@@ -1158,29 +1186,29 @@ Boolean ssh_mapping_put_vl(SshMapping normal_set,
     {
       flag = set->t[h1].h;
       if (flag < 2)
-	{
-	  if (i < 0)
-	    i = h1;
-	  if (flag == 0)
-	    break;
-	  semi_empty_count++;
-	}
+        {
+          if (i < 0)
+            i = h1;
+          if (flag == 0)
+            break;
+          semi_empty_count++;
+        }
       else
-	{
-	  if (flag == h &&
-	      SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
-	    {
-	      i = h1;
-	      break;
-	    }
-	}
+        {
+          if (flag == h &&
+              SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
+            {
+              i = h1;
+              break;
+            }
+        }
       MODULO_ADD(set->size, h1, h2);
       if (h1 == start)
-	{
-	  if (i != -1)
-	    break;
-	  ssh_mapping_fatal();
-	}
+        {
+          if (i != -1)
+            break;
+          ssh_mapping_fatal();
+        }
     }
 
   set->t[i].h = h;
@@ -1188,47 +1216,50 @@ Boolean ssh_mapping_put_vl(SshMapping normal_set,
   if (flag != h)
     {      
       if (i == (int) h1)
-	set->empties--;
+        set->empties--;
       else
-	set->semi_empties--;
+        set->semi_empties--;
 
       /* Allocate a new buffer for key and it's value and copy
-	 the data. */
+         the data. */
       set->t[i].key_length = key_length;
       set->t[i].value_length = value_length;
 
       if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-	ssh_mapping_set_entry_vl(set, i, key, value);
+        ssh_mapping_set_entry_vl(set, i, key, value);
       else
-	{
-	  set->t[i].value = ssh_xmalloc(value_length);
-	  memcpy(set->t[i].value, value, value_length);
-	  if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
-	    set->t[i].key.i = *((unsigned long *)key);
-	  else
-	    {
-	      set->t[i].key.p = ssh_xmalloc(key_length);
-	      memcpy(set->t[i].key.p, key, key_length);
-	    }
-	}
+        {
+          set->t[i].value = ssh_xmalloc(value_length);
+          memcpy(set->t[i].value, value, value_length);
+          if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
+            set->t[i].key.i = *((unsigned long *)key);
+          else
+            {
+              set->t[i].key.p = ssh_xmalloc(key_length);
+              memcpy(set->t[i].key.p, key, key_length);
+            }
+        }
       return FALSE;
     }
 
   /* Mapping for this key already existed. Replace the old value with
      a new one. */
   if (set->flags & SSH_MAPPING_FL_STORE_POINTERS)
-    set->t[i].value = value;
+    {
+      set->t[i].value_length = value_length;
+      ssh_mapping_set_entry_vl(set, i, key, value);
+    }
   else
     {
       if (value_length == set->t[i].value_length)
-	memcpy(set->t[i].value, value, value_length);
+        memcpy(set->t[i].value, value, value_length);
       else
-	{
-	  ssh_xfree(set->t[i].value);
-	  set->t[i].value_length = value_length;
-	  set->t[i].value = ssh_xmalloc(value_length);      
-	  memcpy(set->t[i].value, value, value_length);
-	}
+        {
+          ssh_xfree(set->t[i].value);
+          set->t[i].value_length = value_length;
+          set->t[i].value = ssh_xmalloc(value_length);      
+          memcpy(set->t[i].value, value, value_length);
+        }
     }
   
   return TRUE;
@@ -1239,13 +1270,32 @@ Boolean ssh_mapping_put_vl(SshMapping normal_set,
    pointer from the mapping and it is callers responsibility to
    free it with ssh_xfree. */
 Boolean ssh_mapping_remove_vl(SshMapping normal_set, 
-			      const void *key, size_t key_length,
-			      void **value_return, size_t *value_length)
+                              const void *key, size_t key_length,
+                              void **value_return, size_t *value_length)
+{
+  return ssh_mapping_remove_key_vl(normal_set, key, key_length, 
+                                   value_return, value_length, 
+                                   NULL, NULL);
+}
+
+/* Same as ssh_mapping_remove_vl, but if the mapping is
+   STORE_POINTERS type, this function also returns the original key
+   pointer in 'key_return' (and it's size in 'key_length_return'). 
+   Otherwise they are set to NULL and zero, respectively. */  
+Boolean ssh_mapping_remove_key_vl(SshMapping normal_set, 
+                                  const void *key, size_t key_length,
+                                  void **value_return, size_t *value_length,
+                                  void **key_return, size_t *key_length_return)
 {
   SshMappingVL set = (SshMappingVL) normal_set;
   unsigned long h, h1, h2;
   unsigned long flag, start;
 
+  if (key_return != NULL)
+    (*key_return) = NULL;
+  if (key_length_return != NULL)
+    (*key_length_return) = 0;
+
   if (!(set->flags & SSH_MAPPING_FL_VARIABLE_LENGTH))
     ssh_mapping_fatal();
   
@@ -1256,10 +1306,10 @@ Boolean ssh_mapping_remove_vl(SshMapping normal_set,
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     {
       if (set->hash_function)
-	h = set->hash_function(key, sizeof(unsigned long));
+        h = set->hash_function(key, sizeof(unsigned long));
       else
-	/* In case of integer keys, the key can be used directly. */
-	h = *((unsigned long *)key);
+        /* In case of integer keys, the key can be used directly. */
+        h = *((unsigned long *)key);
     }
   else
     h = set->hash_function(key, key_length);
@@ -1276,13 +1326,13 @@ Boolean ssh_mapping_remove_vl(SshMapping normal_set,
     {
       flag = set->t[h1].h;
       if (flag == h &&
-	  SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
-	break;
+          SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
+        break;
       else if (flag == 0)
-	return FALSE;
+        return FALSE;
       MODULO_ADD(set->size, h1, h2);
       if (start == h1)
-	return 0;
+        return 0;
     }
 
   set->semi_empties++;
@@ -1294,9 +1344,9 @@ Boolean ssh_mapping_remove_vl(SshMapping normal_set,
   else
     {
       /* Store the pointer into context for later retrieval with
-	 ssh_mapping_copy_value_vl(). */
+         ssh_mapping_copy_value_vl(). */
       if (set->remove_value && !(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
-	ssh_xfree(set->cached_value);
+        ssh_xfree(set->cached_value);
       set->cached_value = set->t[h1].value;
       set->cached_length = set->t[h1].value_length;
       set->remove_value = TRUE;
@@ -1304,9 +1354,14 @@ Boolean ssh_mapping_remove_vl(SshMapping normal_set,
   if (value_length)
     *value_length = set->t[h1].value_length;
   
+  if (key_return)
+    *key_return = set->t[h1].key.p;
+  if (key_length_return)
+    *key_length_return = set->t[h1].key_length;
+
   /* If integer keys are not defined, free the key also. */
   if (!(set->flags & (SSH_MAPPING_FL_INTEGER_KEY |
-		      SSH_MAPPING_FL_STORE_POINTERS)))
+                      SSH_MAPPING_FL_STORE_POINTERS)))
     ssh_xfree(set->t[h1].key.p);  
   
   /* Decrease the size of the hash table if it appears 
@@ -1317,17 +1372,17 @@ Boolean ssh_mapping_remove_vl(SshMapping normal_set,
       set->semi_empties > (set->size >> 3))
     {
       if (set->size > SSH_MAPPING_MINIMUM_SIZE &&
-	  set->size - set->empties - set->semi_empties < (set->size >> 4))
-	{
+          set->size - set->empties - set->semi_empties < (set->size >> 4))
+        {
 #ifdef PRIME_TABLE
-	  set->size_shift--;
-	  ssh_mapping_refresh_vl(set, prime_table[set->size_shift]);
+          set->size_shift--;
+          ssh_mapping_refresh_vl(set, prime_table[set->size_shift]);
 #else
-	  ssh_mapping_refresh_vl(set, set->size >> 1);
+          ssh_mapping_refresh_vl(set, set->size >> 1);
 #endif /* PRIME_TABLE */
-	}
+        }
       else  
-	ssh_mapping_refresh_vl(set, set->size);
+        ssh_mapping_refresh_vl(set, set->size);
     }
   
   return TRUE;
@@ -1338,8 +1393,8 @@ Boolean ssh_mapping_remove_vl(SshMapping normal_set,
    value, copies data and returns a pointer to it, along with it's
    size. It is callers responsibility to free it with ssh_xfree. */
 Boolean ssh_mapping_get_vl(const SshMapping normal_set,
-			   const void *key, size_t key_length,
-			   void **value_return, size_t *value_length)
+                           const void *key, size_t key_length,
+                           void **value_return, size_t *value_length)
 {
   SshMappingVL set = (SshMappingVL) normal_set;
   unsigned long h, h1, flag;
@@ -1355,10 +1410,10 @@ Boolean ssh_mapping_get_vl(const SshMapping normal_set,
   if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
     {
       if (set->hash_function)
-	h = set->hash_function(key, sizeof(unsigned long));
+        h = set->hash_function(key, sizeof(unsigned long));
       else
-	/* In case of integer keys, the key can be used directly. */
-	h = *((unsigned long *)key);
+        /* In case of integer keys, the key can be used directly. */
+        h = *((unsigned long *)key);
     }
   else
     h = set->hash_function(key, key_length);
@@ -1375,26 +1430,26 @@ Boolean ssh_mapping_get_vl(const SshMapping normal_set,
       SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
     {
       if (value_return)
-	{
-	  *value_return = set->t[h1].value;
-	}
+        {
+          *value_return = set->t[h1].value;
+        }
       else
-	{
-	  if (set->remove_value &&
-	      !(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
-	    ssh_xfree(set->cached_value);
-	  set->cached_value = set->t[h1].value;
-	  set->cached_length = set->t[h1].value_length;
-	  set->remove_value = FALSE;
-	}
+        {
+          if (set->remove_value &&
+              !(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
+            ssh_xfree(set->cached_value);
+          set->cached_value = set->t[h1].value;
+          set->cached_length = set->t[h1].value_length;
+          set->remove_value = FALSE;
+        }
       if (value_length)
-	*value_length = set->t[h1].value_length;
+        *value_length = set->t[h1].value_length;
       return TRUE;
     }
   else
     {
       if (flag == 0)
-	return FALSE;
+        return FALSE;
     }
   
   start = h1;
@@ -1404,29 +1459,29 @@ Boolean ssh_mapping_get_vl(const SshMapping normal_set,
       MODULO_ADD(set->size, h1, h2);
       flag = set->t[h1].h;
       if (flag == h &&
-	  SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
-	{
-	  /* Return the value pointer if asked. */
-	  if (value_return)
-	    *value_return = set->t[h1].value;
-	  else
-	    {
-	      /* Copy this pointer into context. */
-	      if (set->remove_value &&
-		  !(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
-		ssh_xfree(set->cached_value);
-	      set->cached_value = set->t[h1].value;
-	      set->cached_length = set->t[h1].value_length;
-	      set->remove_value = FALSE;
-	    }
-	  if (value_length)
-	    *value_length = set->t[h1].value_length;
-	  return TRUE;
-	}
+          SSH_MAPPING_COMPARE_KEY_VL(set, h1, key, key_length))
+        {
+          /* Return the value pointer if asked. */
+          if (value_return)
+            *value_return = set->t[h1].value;
+          else
+            {
+              /* Copy this pointer into context. */
+              if (set->remove_value &&
+                  !(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
+                ssh_xfree(set->cached_value);
+              set->cached_value = set->t[h1].value;
+              set->cached_length = set->t[h1].value_length;
+              set->remove_value = FALSE;
+            }
+          if (value_length)
+            *value_length = set->t[h1].value_length;
+          return TRUE;
+        }
       else if (flag == 0)
-	return FALSE;
+        return FALSE;
       if (h1 == start)
-	return FALSE;     
+        return FALSE;     
     }  
 }
 
@@ -1434,8 +1489,8 @@ Boolean ssh_mapping_get_vl(const SshMapping normal_set,
 /* Variable length version of ssh_mapping_get_next. */
 
 Boolean ssh_mapping_get_next_vl(SshMapping normal_set,
-				void **key_return, size_t *key_length,
-				void **value_return, size_t *value_length)
+                                void **key_return, size_t *key_length,
+                                void **value_return, size_t *value_length)
 {
   SshMappingVL set = (SshMappingVL) normal_set;
   unsigned long i;
@@ -1450,44 +1505,44 @@ Boolean ssh_mapping_get_next_vl(SshMapping normal_set,
   for (i = set->idx; i < set->size; i++)
     if (set->t[i].h > 1)
       {
-	/* If a full entry was found, update the index. */
-	set->idx = i + 1;
+        /* If a full entry was found, update the index. */
+        set->idx = i + 1;
 
-	/* If asked, return the key. */
-	if (key_return)
-	  {
-	    if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
-	      *((unsigned long *)key_return) = set->t[i].key.i;
-	    else
-	      *key_return = set->t[i].key.p;
-	  }
-	if (key_length)
-	  {
-	    if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
-	      *key_length = sizeof(unsigned long);
-	    else
-	      *key_length = set->t[i].key_length;
-	  }
+        /* If asked, return the key. */
+        if (key_return)
+          {
+            if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
+              *((unsigned long *)key_return) = set->t[i].key.i;
+            else
+              *key_return = set->t[i].key.p;
+          }
+        if (key_length)
+          {
+            if (set->flags & SSH_MAPPING_FL_INTEGER_KEY)
+              *key_length = sizeof(unsigned long);
+            else
+              *key_length = set->t[i].key_length;
+          }
 
-	/* Return the value, is asked. */
-	if (value_return)
-	  *value_return = set->t[i].value;
-	else
-	  {
-	    /* If value_return was null, store this pointer into a
-	       context for possible retrieval with
-	       ssh_mapping_copy_value_vl() */
-	    if (set->remove_value &&
-		!(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
-	      ssh_xfree(set->cached_value);
-	    set->cached_value = set->t[i].value;
-	    set->cached_length = set->t[i].value_length;
-	    set->remove_value = FALSE;
-	  }
-	if (value_length)
-	  *value_length = set->t[i].value_length;
+        /* Return the value, is asked. */
+        if (value_return)
+          *value_return = set->t[i].value;
+        else
+          {
+            /* If value_return was null, store this pointer into a
+               context for possible retrieval with
+               ssh_mapping_copy_value_vl() */
+            if (set->remove_value &&
+                !(set->flags & SSH_MAPPING_FL_STORE_POINTERS))
+              ssh_xfree(set->cached_value);
+            set->cached_value = set->t[i].value;
+            set->cached_length = set->t[i].value_length;
+            set->remove_value = FALSE;
+          }
+        if (value_length)
+          *value_length = set->t[i].value_length;
 
-	return TRUE;
+        return TRUE;
       }
 
   return FALSE;
@@ -1499,7 +1554,7 @@ Boolean ssh_mapping_get_next_vl(SshMapping normal_set,
    a given buffer. If no value is cached in mapping context or if
    buffers size is wrong, this returns FALSE.  Otherwise TRUE. */
 Boolean ssh_mapping_copy_value_vl(SshMapping normal_set,
-				  void *value_return, size_t value_length)
+                                  void *value_return, size_t value_length)
 {
   SshMappingVL set = (SshMappingVL) normal_set;
   
